"""
SEE COPYRIGHT, LICENCE, and DOCUMENTATION NOTICES: files
README-COPYRIGHT-utf8.txt, README-LICENCE-utf8.txt, and README-DOCUMENTATION-utf8.txt
at project source root.
"""

ISSUE_BASE_CONTEXT = "BASE"

ISSUE_CONTEXT_SEPARATOR = ">"
ISSUE_CONTEXT_END = ":: "
