"""
SEE COPYRIGHT, LICENCE, and DOCUMENTATION NOTICES: files
README-COPYRIGHT-utf8.txt, README-LICENCE-utf8.txt, and README-DOCUMENTATION-utf8.txt
at project source root.
"""

from logger_36.constant.message import LINE_INDENT  # noqa
from logger_36.task.format.message import MessageWithActualExpected  # noqa
