"""
SEE COPYRIGHT, LICENCE, and DOCUMENTATION NOTICES: files
README-COPYRIGHT-utf8.txt, README-LICENCE-utf8.txt, and README-DOCUMENTATION-utf8.txt
at project source root.
"""

from logger_36.config.rule import DEFAULT_RULE_LENGTH, RULE_CHARACTER

MIN_HALF_RULE_LENGTH = 4
DEFAULT_RULE = DEFAULT_RULE_LENGTH * RULE_CHARACTER
